<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WpSecure
 * @subpackage WpSecure/includes
 * @author     WpSecure <contact@WpSecure.com>
 */
class WpSecureBrute {
    private $wpsecure;
    public $is_old_login_page = false;
	
	const ver = WPS_VERSION;
    const slug = WPS_SLUG;
	/**
	 * Initialize the class and set its properties.
	 */
	public function __construct( $wpsecure ) {
		$this->wpsecure = $wpsecure;
		
        include_once('lib/brute.helper.php');
		
		$this->bh = new Brute_Helper(self::slug, self::ver);
		$this->bh->check_versions('1.0', '1.0');	
		
		if(function_exists('add_action')){
			add_action('plugins_loaded', array( $this, 'loginizer_load_plugin') );
		}		
	}


	function loginizer_load_plugin(){
		
		global $loginizer;
		
		// Set the array
		$loginizer = array();
		
		$loginizer['prefix'] = !defined('SITEPAD') ? 'Loginizer ' : 'SitePad ';
		$loginizer['app'] = !defined('SITEPAD') ? 'WordPress' : 'SitePad';
		$loginizer['login_basename'] = !defined('SITEPAD') ? 'wp-login.php' : 'login.php';
		$loginizer['wp-includes'] = !defined('SITEPAD') ? 'wp-includes' : 'site-inc';
		
		// The IP Method to use
		$loginizer['ip_method'] = get_option('loginizer_ip_method');
		if($loginizer['ip_method'] == 3){
			$loginizer['custom_ip_method'] = get_option('loginizer_custom_ip_method');
		}
		
		// Load settings
		$options = get_option('wps_options',false);
		$loginizer['max_retries'] = empty($options['max_retries']) ? 3 : $options['max_retries'];
		$loginizer['lockout_time'] = empty($options['lockout_time']) ? 900 : $options['lockout_time']; // 15 minutes
		$loginizer['max_lockouts'] = empty($options['max_lockouts']) ? 5 : $options['max_lockouts'];
		$loginizer['lockouts_extend'] = empty($options['lockouts_extend']) ? 86400 : $options['lockouts_extend']; // 24 hours
		$loginizer['reset_retries'] = empty($options['reset_retries']) ? 86400 : $options['reset_retries']; // 24 hours
		$loginizer['notify_email'] = empty($options['notify_email']) ? 0 : $options['notify_email'];
		$loginizer['notify_email_address'] = $this->bh->lz_is_multisite() ? get_site_option('admin_email') : get_option('admin_email');
		
		if(!empty($options['notify_email_address'])){
			$loginizer['notify_email_address'] = $options['notify_email_address'];
			$loginizer['custom_notify_email'] = 1;
		}
		
		// Default messages
		$loginizer['d_msg']['inv_userpass'] = __('Incorrect Username or Password', 'loginizer');
		$loginizer['d_msg']['ip_blacklisted'] = __('Your IP has been blacklisted', 'loginizer');
		$loginizer['d_msg']['attempts_left'] = __('attempt(s) left', 'loginizer');
		$loginizer['d_msg']['lockout_err'] = __('You have exceeded maximum login retries<br /> Please try after', 'loginizer');
		$loginizer['d_msg']['minutes_err'] = __('minute(s)', 'loginizer');
		$loginizer['d_msg']['hours_err'] = __('hour(s)', 'loginizer');
		
		// Message Strings
		$loginizer['msg'] = get_option('loginizer_msg');
		
		foreach($loginizer['d_msg'] as $lk => $lv){
			if(empty($loginizer['msg'][$lk])){
				$loginizer['msg'][$lk] = $loginizer['d_msg'][$lk];
			}
		}
		
		// When was the database cleared last time
		$loginizer['last_reset']  = get_option('loginizer_last_reset');
		
		//print_r($loginizer);
		
		// Clear retries
		if((time() - $loginizer['last_reset']) >= $loginizer['reset_retries']){
			$this->loginizer_reset_retries();
		}
		
		$ins_time = get_option('loginizer_ins_time');
		if(empty($ins_time)){
			$ins_time = time();
			update_option('loginizer_ins_time', $ins_time);
		}
		$loginizer['ins_time'] = $ins_time;
		
		// Set the current IP
		$loginizer['current_ip'] = $this->bh->lz_getip();
		
		// Is Brute Force Disabled ?
		//$loginizer['disable_brute'] = get_option('loginizer_disable_brute');
		$loginizer['disable_brute'] =false;
		// Filters and actions
		if(empty($loginizer['disable_brute'])){
		
			// Use this to verify before WP tries to login
			// Is always called and is the first function to be called
			//add_action('wp_authenticate', 'loginizer_wp_authenticate', 10, 2);// Not called by XML-RPC
			add_filter('authenticate', array( $this, 'loginizer_wp_authenticate'), 10001, 3);// This one is called by xmlrpc as well as GUI
			
			// Is called when a login attempt fails
			// Hence Update our records that the login failed
			add_action('wp_login_failed', array( $this, 'loginizer_login_failed'));
			
			// Is called before displaying the error message so that we dont show that the username is wrong or the password
			// Update Error message
			add_action('wp_login_errors', array( $this, 'loginizer_error_handler'), 10001, 2);
			add_action('woocommerce_login_failed', array( $this, 'loginizer_woocommerce_error_handler'), 10001);
		
		}
		
		// ----------------
		// PRO INIT
		// ----------------
		
		// Email to Login
		$options = get_option('loginizer_epl');
		$loginizer['pl_d_sub'] = 'Login at $site_name';
		$loginizer['pl_d_msg'] = 'Hi,

	A login request was submitted for your account $email at :
	$site_name - $site_url

	Login at $site_name by visiting this url : 
	$login_url

	If you have not requested for the Login URL, please ignore this email.

	Regards,
	$site_name';
		$loginizer['email_pass_less'] = empty($options['email_pass_less']) ? 0 : $options['email_pass_less'];
		$loginizer['passwordless_sub'] = empty($options['passwordless_sub']) ? $loginizer['pl_d_sub'] : $options['passwordless_sub'];
		$loginizer['passwordless_msg'] = empty($options['passwordless_msg']) ? $loginizer['pl_d_msg'] : $options['passwordless_msg'];
		$loginizer['passwordless_msg_is_custom'] = empty($options['passwordless_msg']) ? 0 : 1;
		$loginizer['passwordless_html'] = empty($options['passwordless_html']) ? 0 : $options['passwordless_html'];
		
		// For SitePad its always on
		if(defined('SITEPAD')){
			$loginizer['email_pass_less'] = 1;
		}

		// Auto Blacklist Usernames
		$loginizer['username_blacklist'] = get_option('loginizer_username_blacklist');
		
		$loginizer['domains_blacklist'] = get_option('loginizer_domains_blacklist');
		
		$loginizer['wp_admin_d_msg'] = __('LZ : Not allowed via WP-ADMIN. Please access over the new Admin URL', 'loginizer');
		

	}



	// Should return NULL if everything is fine
	function loginizer_wp_authenticate($user, $username, $password){
		
		global $loginizer, $lz_error, $lz_cannot_login, $lz_user_pass;
		
		if(!empty($username) && !empty($password)){
			$lz_user_pass = 1;
		}
		

		// Is the username blacklisted ?
		if(function_exists('loginizer_user_blacklisted')){
			if(loginizer_user_blacklisted($username)){
				$lz_cannot_login = 1;
			
				// This is used by WP Activity Log
				apply_filters( 'wp_login_blocked', $username );
				
				return new WP_Error('user_blacklisted', implode('', $lz_error), 'loginizer');
			}
		}
		
		if($this->loginizer_can_login()){
			return $user;
		}
		
		$lz_cannot_login = 1;
			
		// This is used by WP Activity Log
		apply_filters( 'wp_login_blocked', $username );
		
		return new WP_Error('ip_blocked', implode('', $lz_error), 'loginizer');
		
	}

	function loginizer_can_login(){
		
		global $wpdb, $loginizer, $lz_error;
		
		// Get the logs
		$sel_query = $wpdb->prepare("SELECT * FROM `".$wpdb->prefix."secure_brute_logs` WHERE `ip` = %s", $loginizer['current_ip']);
		$result = $this->bh->lz_selectquery($sel_query);
		
		if(!empty($result['count']) && ($result['count'] % $loginizer['max_retries']) == 0){
			
			// Has he reached max lockouts ?
			if($result['lockout'] >= $loginizer['max_lockouts']){
				$loginizer['lockout_time'] = $loginizer['lockouts_extend'];
			}
			
			// Is he in the lockout time ?
			if($result['time'] >= (time() - $loginizer['lockout_time'])){
				$banlift = ceil((($result['time'] + $loginizer['lockout_time']) - time()) / 60);
				
				//echo 'Current Time '.date('d/M/Y H:i:s P', time()).'<br />';
				//echo 'Last attempt '.date('d/M/Y H:i:s P', $result['time']).'<br />';
				//echo 'Unlock Time '.date('d/M/Y H:i:s P', $result['time'] + $loginizer['lockout_time']).'<br />';
				
				$_time = $banlift.' '.$loginizer['msg']['minutes_err'];
				
				if($banlift > 60){
					$banlift = ceil($banlift / 60);
					$_time = $banlift.' '.$loginizer['msg']['hours_err'];
				}
				
				$lz_error['ip_blocked'] = $loginizer['msg']['lockout_err'].' '.$_time;
				
				return false;
			}
		}
		
		return true;
	}

	function loginizer_is_blacklisted(){
		
		global $wpdb, $loginizer, $lz_error;
		
		$blacklist = $loginizer['blacklist'];
		
		if(empty($blacklist)){
			return false;
		}
		  
		foreach($blacklist as $k => $v){
			
			// Is the IP in the blacklist ?
			if(inet_ptoi($v['start']) <= inet_ptoi($loginizer['current_ip']) && inet_ptoi($loginizer['current_ip']) <= inet_ptoi($v['end'])){
				$result = 1;
				break;
			}
			
			// Is it in a wider range ?
			if(inet_ptoi($v['start']) >= 0 && inet_ptoi($v['end']) < 0){
				
				// Since the end of the RANGE (i.e. current IP range) is beyond the +ve value of inet_ptoi, 
				// if the current IP is <= than the start of the range, it is within the range
				// OR
				// if the current IP is <= than the end of the range, it is within the range
				if(inet_ptoi($v['start']) <= inet_ptoi($loginizer['current_ip'])
					|| inet_ptoi($loginizer['current_ip']) <= inet_ptoi($v['end'])){				
					$result = 1;
					break;
				}
				
			}
			
		}
			
		// You are blacklisted
		if(!empty($result)){
			$lz_error['ip_blacklisted'] = $loginizer['msg']['ip_blacklisted'];
			return true;
		}
		
		return false;
		
	}

	function loginizer_is_whitelisted(){
		
		global $wpdb, $loginizer, $lz_error;
		
		$whitelist = $loginizer['whitelist'];
				
		if(empty($whitelist)){
			return false;
		}
		  
		foreach($whitelist as $k => $v){
			
			// Is the IP in the blacklist ?
			if(inet_ptoi($v['start']) <= inet_ptoi($loginizer['current_ip']) && inet_ptoi($loginizer['current_ip']) <= inet_ptoi($v['end'])){
				$result = 1;
				break;
			}
			
			// Is it in a wider range ?
			if(inet_ptoi($v['start']) >= 0 && inet_ptoi($v['end']) < 0){
				
				// Since the end of the RANGE (i.e. current IP range) is beyond the +ve value of inet_ptoi, 
				// if the current IP is <= than the start of the range, it is within the range
				// OR
				// if the current IP is <= than the end of the range, it is within the range
				if(inet_ptoi($v['start']) <= inet_ptoi($loginizer['current_ip'])
					|| inet_ptoi($loginizer['current_ip']) <= inet_ptoi($v['end'])){				
					$result = 1;
					break;
				}
				
			}
			
		}
			
		// You are whitelisted
		if(!empty($result)){
			return true;
		}
		
		return false;
		
	}


	// When the login fails, then this is called
	// We need to update the database
	function loginizer_login_failed($username, $is_2fa = ''){
		
		global $wpdb, $loginizer, $lz_cannot_login;
		
		// Some plugins are changing the value for username as null so we need to handle it before using it for the INSERT OR UPDATE query
		if(empty($username) || is_null($username)){
			$username = '';
		}
		
		$fail_type = 'Login';
		
		if(!empty($is_2fa)){
			$fail_type = '2FA';
		}

		if(empty($lz_cannot_login) && empty($loginizer['ip_is_whitelisted']) && empty($loginizer['no_loginizer_logs'])){
			
			$url = @addslashes((!empty($_SERVER['HTTPS']) ? 'https://' : 'http://').$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);
			$url = esc_url($url);
			
			$sel_query = $wpdb->prepare("SELECT * FROM `".$wpdb->prefix."secure_brute_logs` WHERE `ip` = %s", $loginizer['current_ip']);
			$result = $this->bh->lz_selectquery($sel_query);
			
			if(!empty($result)){
				$lockout = floor((($result['count']+1) / $loginizer['max_retries']));
				
				$update_data = array('username' => $username, 
									'time' => time(), 
									'count' => $result['count']+1, 
									'lockout' => $lockout, 
									'url' => $url);
				
				$where_data = array('ip' => $loginizer['current_ip']);
				
				$format = array('%s','%d','%d','%d','%s');
				$where_format = array('%s');
				
				$wpdb->update($wpdb->prefix.'secure_brute_logs', $update_data, $where_data, $format, $where_format);
				
				// Do we need to email admin ?
				if(!empty($loginizer['notify_email']) && $lockout >= $loginizer['notify_email']){
					
					$lockout_time = $loginizer['lockout_time'];
					
					if($lockout >= $loginizer['max_lockouts']){
						// extended lockout is in hours so we have to convert to minute
						$lockout_time = $loginizer['lockouts_extend'];
					}
					
					$sitename = lz_is_multisite() ? get_site_option('site_name') : get_option('blogname');
					$mail = array();
					$mail['to'] = $loginizer['notify_email_address'];	
					$mail['subject'] = 'Failed '.$fail_type.' Attempts from IP '.$loginizer['current_ip'].' ('.$sitename.')';
					$mail['message'] = 'Hi,

	'.($result['count']+1).' failed '.strtolower($fail_type).' attempts and '.$lockout.' lockout(s) from IP '.$loginizer['current_ip'].' on your site :
	'.home_url().'

	Last '.$fail_type.' Attempt : '.date('d/M/Y H:i:s P', time()).'
	Last User Attempt : '.$username.'
	IP has been blocked until : '.date('d/M/Y H:i:s P', time() + $lockout_time).'

	Regards,
	Loginizer';

					@wp_mail($mail['to'], $mail['subject'], $mail['message']);
				}
			}else{
				$result = array();
				$result['count'] = 0;
				
				$insert_data = array('username' => $username, 
									'time' => time(), 
									'count' => 1, 
									'ip' => $loginizer['current_ip'], 
									'lockout' => 0, 
									'url' => $url);
									
				$format = array('%s','%d','%d','%s','%d','%s');
				
				$wpdb->insert($wpdb->prefix.'secure_brute_logs', $insert_data, $format);
			}
		
			// We need to add one as this is a failed attempt as well
			$result['count'] = $result['count'] + 1;
			$loginizer['retries_left'] = ($loginizer['max_retries'] - ($result['count'] % $loginizer['max_retries']));
			$loginizer['retries_left'] = $loginizer['retries_left'] == $loginizer['max_retries'] ? 0 : $loginizer['retries_left'];
			
		}
	}

	// Handles the error of the password not being there
	function loginizer_error_handler($errors, $redirect_to){
		
		global $wpdb, $loginizer, $lz_user_pass, $lz_cannot_login;
		
		//echo 'loginizer_error_handler :';print_r($errors->errors);echo '<br>';
		
		// Remove the empty password error
		if(is_wp_error($errors)){
			
			$codes = $errors->get_error_codes();
			
			foreach($codes as $k => $v){
				if($v == 'invalid_username' || $v == 'incorrect_password'){
					$show_error = 1;
				}
			}
			
			$errors->remove('invalid_username');
			$errors->remove('incorrect_password');
			
		}
		
		// Add the error
		if(!empty($lz_user_pass) && !empty($show_error) && empty($lz_cannot_login)){
			$errors->add('invalid_userpass', '<b>ERROR:</b> ' . $loginizer['msg']['inv_userpass']);
		}
		
		// Add the number of retires left as well
		if(count($errors->get_error_codes()) > 0 && isset($loginizer['retries_left'])){
			$errors->add('retries_left', $this->loginizer_retries_left());
		}
		
		return $errors;
		
	}



	// Handles the error of the password not being there
	function loginizer_woocommerce_error_handler(){
		
		global $wpdb, $loginizer, $lz_user_pass, $lz_cannot_login;
		
		if(function_exists('wc_add_notice')){
			wc_add_notice( $this->loginizer_retries_left(), 'error' );
		}
		
	}

	// Returns a string with the number of retries left
	function loginizer_retries_left(){
		
		global $wpdb, $loginizer, $lz_user_pass, $lz_cannot_login;
		
		// If we are to show the number of retries left
		if(isset($loginizer['retries_left'])){
			return '<b>'.$loginizer['retries_left'].'</b> '.$loginizer['msg']['attempts_left'];
		}
		
	}

	function loginizer_reset_retries(){
		
		global $wpdb, $loginizer;
		
		$deltime = time() - $loginizer['reset_retries'];
		
		$del_query = $wpdb->prepare("DELETE FROM `".$wpdb->prefix."secure_brute_logs` WHERE `time` <= %d", $deltime);
		$result = $wpdb->query($del_query);
		
		update_option('loginizer_last_reset', time());
		
	}
	
}
